//------------------------------------------------------
//
//   Greg's Blocks Mod - Packet handler
//
//------------------------------------------------------

package gcewing.blocks;

import java.io.*;

import net.minecraft.src.*;
import cpw.mods.fml.common.network.*;

public class PacketHandler implements IPacketHandler {

	public static final String channelName = "gce.blocks";
	static final byte sawbenchSelectShapePkt = 1;
	static final byte tileEntityPkt = 2;
	
	public static Packet250CustomPayload newPacket250(byte[] data) {
		Packet250CustomPayload pkt = new Packet250CustomPayload();
		pkt.channel = channelName;
		pkt.length = data.length;
		pkt.data = data;
		return pkt;
	}
	
	public static void sendSawbenchSelectShape(int i) {
		sendDataToServer(new byte[] {sawbenchSelectShapePkt, (byte)i});
	}
	
	static void sendDataToServer(byte[] data) {
		Packet250CustomPayload pkt = newPacket250(data);
		System.out.printf("mod_GregsBlocks: Client sending packet on channel %s\n",
			pkt.channel);
		//System.out.printf("mod_GregsBlocks: Data =");
		//for (int i = 0; i < pkt.length; i++)
		//	System.out.printf(" %02X", pkt.data[i]);
		//System.out.printf("\n");
		PacketDispatcher.sendPacketToServer(pkt);
	}

	@Override 
	public void onPacketData(INetworkManager manager, Packet250CustomPayload pkt, Player player) {
//		System.out.printf("GregsBlocks: received packet on channel %s\n",
//			pkt.channel);
		byte[] data = pkt.data;
		switch (data[0]) {
			case sawbenchSelectShapePkt:
				handleSawbenchSelectShape(player, data[1]);
				break;
			case tileEntityPkt:
				handleTileEntityPacket(pkt);
				break;
		}
	}

	void handleSawbenchSelectShape(Player player, int value) {
		Container ci = ((EntityPlayer)player).craftingInventory;
//		System.out.printf("GregsBlocks: player.craftingInventory = %s\n",
//			player.craftingInventory);
		if (ci instanceof ContainerSawbench)
			((ContainerSawbench)ci).setSelectedShape(value);
	}

	void handleTileEntityPacket(Packet250CustomPayload pkt) {
		//System.out.printf("GregsBlocks: PacketHandler.handleTileEntityPacket\n");
		NBTTagCompound tag = nbtFromPacket(pkt);
		int x = tag.getInteger("x");
		int y = tag.getInteger("y");
		int z = tag.getInteger("z");
		World world = GregsBlocks.proxy.getWorld();
		if (world.blockExists(x, y, z)) {
			TileEntity te = world.getBlockTileEntity(x, y, z);
			//System.out.printf("GregsBlocks: PacketHandler received tileEntityPkt: %d %d %d block %d %s <- %s\n",
			//	x, y, z, world.getBlockId(x, y, z), te, tag.getString("id"));
			if (te != null) {
				te.readFromNBT(tag);
				te.onInventoryChanged();
			}
		}
	}

	static NBTTagCompound nbtFromPacket(Packet250CustomPayload pkt) {
		DataInput stream = new DataInputStream(new ByteArrayInputStream(pkt.data, 1, pkt.length - 1));
		try {
			NBTBase tag = NBTBase.readNamedTag(stream);
			return (NBTTagCompound)tag;
		}
		catch (IOException e) {
			throw new RuntimeException(e.toString());
		}
	}
	
	public static Packet packetFromTileEntity(TileEntity te) {
		NBTTagCompound tag = new NBTTagCompound();
		te.writeToNBT(tag);
		Packet pkt = packetFromNBT(tileEntityPkt, tag);
		pkt.isChunkDataPacket = true;
		return pkt;
	}
	
	static Packet packetFromNBT(int type, NBTTagCompound tag) {
		ByteArrayOutputStream bytes = new ByteArrayOutputStream();
		DataOutputStream stream = new DataOutputStream(bytes);
		try {
			stream.write(type);
			NBTBase.writeNamedTag(tag, stream);
		}
		catch (IOException e) {
			throw new RuntimeException(e.toString());
		}
		byte[] data = bytes.toByteArray();
		Packet250CustomPayload packet = newPacket250(data);
		//System.out.printf("PacketHandler.packetFromNBT: channel %s length %d\n",
		//	packet.channel, packet.length);
		return packet;
	}
	
}
